require "restful_celluloid/version"
require 'webrick'
require 'sinatra/base'
require 'thread'
require 'celluloid'
require 'json'

module RestfulCelluloid


  def self.start! port
    RESTfulServer.port=port
    RESTfulServer.run!
  end

  def self.start port
    @port=port
    RESTfulServer.port=port
    server_queue=Queue.new
    Thread.new do
      RESTfulServer.run! do |server|
        server_queue << server
      end
    end
    @server=server_queue.pop
  end

  def self.stop
    @server.shutdown
  end

  class RESTfulServer < ::Sinatra::Base
    set :server, %w[webrick]
    set :server_settings, Logger: WEBrick::Log.new("/dev/null")

    get '/actors' do
      content_type :json
      Celluloid::Actor.all.map{|actor| actor.name}.to_json
    end

    get '/actor/:actor_name/:method_name' do
      content_type :json
      actor_name=params[:actor_name]
      method_name=params[:method_name]
      actor=Celluloid::Actor[actor_name]
      if actor        
        if actor.respond_to? method_name
          begin
          params=request.env['rack.request.query_hash']
          result=params.empty? ? actor.send(method_name) : actor.send(method_name,params)
          result.to_json
        rescue => ex
          status 500
          {error: 'Method execution raised exception.', message: ex.message, backtrace: ex.backtrace}.to_json
        end
        else
        status 404
        return {error: ("Method '%s' not found on actor '%s'." % [method_name, actor_name])}.to_json          
        end
      else
        status 404
        return {error: ("Actor '%s' not found." % actor_name)}.to_json
      end
    end


  end

end
